-------------------------------------------------------------------------------
--
-- Title       : context_filter_TB
-- Design      : filtr_obrazu
-- Author      : Pawe Sokoowski
-- Company     : AGH
--
-------------------------------------------------------------------------------
--
-- Description : TestBench module
--
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

entity context_filter_tb is
end context_filter_tb;

architecture TB_ARCHITECTURE of context_filter_tb is 
	
	-- constants for components
	constant DEPTH : integer := 9;				-- number of bits for fifo addressing
	constant H_DIM : integer := 512;			-- horizontal dimension of the image
	constant V_DIM : integer := 512;			-- vertical dimension of the image
	constant D_DIM : integer := 3; 				-- number of parts of the image (3 for rgb)
	constant NO_FRAMES : integer := 1; 			-- number of frames
	constant PIPELINE_LENGTH : integer := 512;	-- length of pipeline in filter procedure
	
	-- Component declaration of the tested unit
	component context_filter
		generic(
			DEPTH : INTEGER ;
			H_DIM : INTEGER ;
			V_DIM : INTEGER 
			);
		port(
			CLK : in STD_LOGIC;
			CLR : in STD_LOGIC;
			EN : in STD_LOGIC;
			PIXEL_IN : in STD_LOGIC_VECTOR(7 downto 0);
			PIXEL_OUT : out STD_LOGIC_VECTOR(7 downto 0) 
			);
	end component;
	
	
	-- auxiliary component declaration
	
	component Get_Image is
		generic (
			constant V_IM_DIM  : integer ;		-- maximum vertical size of image
			constant H_IM_DIM  : integer ;		-- maximum horizontal size of image
			constant D_IM_DIM  : integer ; 		-- number of parts of the image (3 for rgb)
			constant NO_FRAMES : integer 		-- number of frames
			);
		port(
			CLK : in STD_LOGIC;
			CLR : in STD_LOGIC;
			DATA_OUT_EN : out STD_LOGIC;
			DATA_OUT : out STD_LOGIC_VECTOR(7 downto 0)
			);
	end component Get_Image;
	
	component Send_Image is
		generic (
			constant V_IM_DIM  : integer ;			-- size of vertical dimension
			constant H_IM_DIM  : integer ;			-- size of horizontal dimension
			constant D_IM_DIM  : integer ;			-- depth of image
			constant NO_FRAMES : integer ;			-- number of frames
			constant PIPELINE_LENGTH : integer		-- length of pipeline
			);
		port(
			CLK : in STD_LOGIC;
			CLR : in STD_LOGIC;
			EN : in STD_LOGIC;
			DATA_IN : in STD_LOGIC_VECTOR(7 downto 0)
			);
	end component Send_Image;
	
	-----------------------------
	
	-- Stimulus signals - signals mapped to the input and inout ports of tested entity
	signal CLK : STD_LOGIC;
	signal CLR : STD_LOGIC;
	signal EN : STD_LOGIC;
	signal PIXEL_IN : STD_LOGIC_VECTOR(7 downto 0);
	-- Observed signals - signals mapped to the output ports of tested entity
	signal PIXEL_OUT : STD_LOGIC_VECTOR(7 downto 0);
	
	--Signal is used to stop clock signal generators
	signal END_SIM: BOOLEAN:=FALSE;
	
begin
	
	-- Unit Under Test port map
	UUT : context_filter
	generic map (
		DEPTH => DEPTH,
		H_DIM => H_DIM,
		V_DIM => V_DIM
		)
	
	port map (
		CLK => CLK,
		CLR => CLR,
		EN => EN,
		PIXEL_IN => PIXEL_IN,
		PIXEL_OUT => PIXEL_OUT
		);
	
	-- Unit which generates stimuluses	
	U1: Get_Image 
	generic map (
		V_IM_DIM  => V_DIM,			-- size of vertical dimension
		H_IM_DIM  => H_DIM,			-- size of horizontal dimension
		D_IM_DIM  => D_DIM,			-- depth of image
		NO_FRAMES => NO_FRAMES		-- number of frames
		)
	port map (
		CLK => CLK,
		CLR => CLR,
		DATA_OUT_EN => EN,
		DATA_OUT => PIXEL_IN
		);
	
	-- Unit which receives output signals
	U2: Send_Image 
	generic map (
		V_IM_DIM  => V_DIM,					-- size of vertical dimension
		H_IM_DIM  => H_DIM,					-- size of horizontal dimension
		D_IM_DIM  => D_DIM,					-- depth of image
		NO_FRAMES => NO_FRAMES,				-- number of frames
		PIPELINE_LENGTH => PIPELINE_LENGTH 	-- length of pipeline
		)
	port map (
		CLK => CLK,
		CLR => CLR,
		EN => EN,
		DATA_IN => PIXEL_OUT
		);
	
	-- signal CLR generation
	CLEAR_PROC: process
	begin  
		CLR <= '1';
		wait for 13 ns; --0 fs
		CLR <= '0';
		wait;
	end process;
	
	-- signal END_SIM generation
	TIME_OF_SIMULATION: process
	begin
		 wait for 7879800 ns;
		 END_SIM <= TRUE;
		--	end of stimulus events
		wait;
	end process;
	
	-- signal CLK generation
	CLOCK_CLK : process
	begin
		if END_SIM = FALSE then
			CLK <= '0';
			wait for 5 ns; --0 fs
		else
			wait;
		end if;
		if END_SIM = FALSE then
			CLK <= '1';
			wait for 5 ns; --5 ns
		else
			wait;
		end if;
	end process;
	
end TB_ARCHITECTURE;

configuration TESTBENCH_FOR_context_filter of context_filter_tb is
	for TB_ARCHITECTURE
		for UUT : context_filter
			use entity work.context_filter(context_filter);
		end for;
	end for;
end TESTBENCH_FOR_context_filter;

